import xml.etree.ElementTree as ET
import os.path
from model.chat import Chat
import manager

'''
Inserta un chat en el XML correspondiente como un nuevo elemento
@param chat: objeto chat que se va a insertar
'''
def insert(chat):
    if not os.path.isfile('data/chats.xml'):
        root = ET.Element('chats')
        
    else:
        tree = ET.parse('data/chats.xml')
        root = tree.getroot()
        
    s = ET.SubElement(root, 'chat')
    ET.SubElement(s, 'id').text = str(chat.getId())
    ET.SubElement(s, 'language').text = chat.getLanguage()
    members = ET.SubElement(s, 'members')
    
    for m in chat.getMembers():
        ET.SubElement(members, 'id').text = str(m)
       
    manager.indent(root)
    tree = ET.ElementTree(root)
    with open('data/chats.xml', 'wb') as file:
        tree.write(file)
        file.close()
 
'''
Busca y retorna todos los objetos Chat almacenados en el XML
@return chats: array de objetos Chat con todos los chats guardados
'''
def findAll():
    chats = {}
    if os.path.isfile('data/chats.xml'):
        with open('data/chats.xml', 'r') as file:
            tree = ET.parse(file)
            root = tree.getroot()
            for child in root.findall('chat'):
                chat_id = child.find('id').text
                language = child.find('language').text
                members = child.find('members')
                mList = []
                for m in members.findall('id'):
                    idM = m.text
                    try:
                        mList.append(int(idM))
                    except ValueError:
                        mList.append(idM)
                
                chat = Chat(int(chat_id))
                chat.setMembers(mList)
                chat.setLanguage(language)
                chats[int(chat_id)] = chat
               
            file.close()
            
    return chats
    
'''
Localiza y actualiza la informacion relativa al lenguaje de un objeto Chat en el XML
@param chat: objeto chat que se va a actualizar
'''
def updateLanguageChat(chat):
    tree = ET.parse('data/chats.xml')
    root = tree.getroot()
    for child in root.findall('chat'):
        if child.find('id').text == str(chat.getId()):
            child.find('language').text = chat.getLanguage()
    
    tree = ET.ElementTree(root)
    with open('data/chats.xml', 'wb') as file:
        tree.write(file)
        file.close()
        
'''
Localiza y actualiza la informacion relativa a los miembros de un objeto Chat en el XML
@param chat: objeto chat que se va a actualizar
'''
def updateMembersChat(chat):
    tree = ET.parse('data/chats.xml')
    root = tree.getroot()
    for child in root.findall('chat'):
        if child.find('id').text == str(chat.getId()):
            members = child.find('members')
            for m in members.findall('id'):
                members.remove(m)
            #Vuelvo a crearlas con sus modificaciones, nuevo inclusive
            for m in chat.getMembers():
                ET.SubElement(members, 'id').text = str(m)
    
    manager.indent(root)
    tree = ET.ElementTree(root)
    with open('data/chats.xml', 'wb') as file:
        tree.write(file)
        file.close()