import xml.etree.ElementTree as ET
import os.path
from model.checklist import Checklist
import manager

'''
Inserta una checklist en el XML correspondiente como un nuevo elemento
@param checklist: objeto Checklist que se va a insertar
'''
def insert(checklist):
    if not os.path.isfile('data/checklists.xml'):
        root = ET.Element('checklists')
        
    else:
        tree = ET.parse('data/checklists.xml')
        root = tree.getroot()
        
    cl = ET.SubElement(root, 'checklist')
    ET.SubElement(cl, 'id').text = str(checklist.getId())
    ET.SubElement(cl, 'name').text = checklist.getName()
    ET.SubElement(cl, 'owner').text = str(checklist.getOwner())
    ET.SubElement(cl, 'state').text = str(checklist.getState())
    for msg in checklist.getMessages():
        ET.SubElement(cl, 'msg').text = str(msg)
    
    manager.indent(root)
    tree = ET.ElementTree(root)
    with open('data/checklists.xml', 'wb') as file:
        tree.write(file)
        file.close()
 
'''
Busca y retorna todos los objetos Checklist almacenados en el XML
@return checklists: array de objetos Checklist con todas las checklists guardadas
'''
def findAll():
    checklists = []
    if os.path.isfile('data/checklists.xml'):
        with open('data/checklists.xml', 'r') as file:
            tree = ET.parse(file)
            root = tree.getroot()
            for child in root.findall('checklist'):
                idList = child.find('id').text
                name = child.find('name').text
                ownerList = child.find('owner').text
                msgList =[]
                for msg in child.findall('msg'):
                    try:
                        m = msg.text.split(',')
                        message = (int(m[0][1:]), int(m[1][:-1]))
                    except ValueError:
                        message = msg.text
                        
                    msgList.append(message)
                state = child.find('state').text
                cl = Checklist(name, int(ownerList), int(idList))
                cl.setMessages(msgList)
                cl.setState(state)
                checklists.append(cl)
                
            file.close()
            
    return checklists
    
'''
Busca y retorna todos los objetos Checklist del creador indicado almacenados en el XML
@param _id: identificador del usuario del cual se buscan sus listas
@return checklists: array de objetos Checklist con todas las checklists guardadas de ese creador
'''
def findAllByOwner(_id):
    checklists = []
    if os.path.isfile('data/checklists.xml'):
        with open('data/checklists.xml', 'r') as file:
            tree = ET.parse(file)
            root = tree.getroot()
            for child in root.findall('checklist'):
                if child.find('owner').text == str(_id):
                    idList = child.find('id').text
                    name = child.find('name').text
                    ownerList = child.find('owner').text
                    msgList =[]
                    for msg in child.findall('msg'):
                        try:
                            m = msg.text.split(',')
                            message = (int(m[0][1:]), int(m[1][:-1]))
                        except ValueError:
                            message = msg.text
                            
                        msgList.append(message)
                    state = child.find('state').text
                    cl = Checklist(name, int(ownerList), int(idList))
                    cl.setMessages(msgList)
                    cl.setState(state)
                    checklists.append(cl)
                
            file.close()
            
    return checklists
    
'''
Busca y retorna el objeto Checklist con el nombre y creador indicados de entre los almacenados en el XML
@param name: nombre de la checklist buscada
@param _id: identificador del usuario creador
@return checklist: objeto Checklist con la checklist coincidente
'''
def findByNameAndOwner(name, _id):
    checklist = None
    if os.path.isfile('data/checklists.xml'):
        with open('data/checklists.xml', 'r') as file:
            tree = ET.parse(file)
            root = tree.getroot()
            for child in root.findall('checklist'):
                if child.find('name').text == str(name) and child.find('owner').text == str(_id):
                    idList = child.find('id').text
                    name = child.find('name').text
                    ownerList = child.find('owner').text
                    msgList =[]
                    for msg in child.findall('msg'):
                        try:
                            m = msg.text.split(',')
                            message = (int(m[0][1:]), int(m[1][:-1]))
                        except ValueError:
                            message = msg.text
                            
                        msgList.append(message)
                    state = child.find('state').text
                    checklist = Checklist(name, int(ownerList), int(idList))
                    checklist.setMessages(msgList)
                    checklist.setState(state)
                    break
                
            file.close()
            
    return checklist
    
'''
Busca y retorna el objeto Checklist con el id indicado de entre los almacenados en el XML
@param _id: identificador de la checklist buscada
@return checklist: objeto Checklist con la checklist coincidente
'''
def findById(_id):
    checklist = None
    if os.path.isfile('data/checklists.xml'):
        with open('data/checklists.xml', 'r') as file:
            tree = ET.parse(file)
            root = tree.getroot()
            for child in root.findall('checklist'):
                if child.find('id').text == str(_id):
                    idList = child.find('id').text
                    name = child.find('name').text
                    ownerList = child.find('owner').text
                    msgList =[]
                    for msg in child.findall('msg'):
                        try:
                            m = msg.text.split(',')
                            message = (int(m[0][1:]), int(m[1][:-1]))
                        except ValueError:
                            message = msg.text
                            
                        msgList.append(message)
                    state = child.find('state').text
                    checklist = Checklist(name, int(ownerList), int(idList))
                    checklist.setMessages(msgList)
                    checklist.setState(state)
                    break
                
            file.close()
            
    return checklist

'''
Busca y actualiza el objeto Checklist indicado en el XML
@param checklist: objeto Checklist que se va a actualizar
'''  
def update(checklist):
    tree = ET.parse('data/checklists.xml')
    root = tree.getroot()
    for child in root.findall('checklist'):
        if child.find('id').text == str(checklist.getId()):
            child.find('state').text = str(checklist.getState())
            #Vacio los msg
            for msg in child.findall('msg'):
                child.remove(msg)
            #Vuelvo a crearlos
            for msg in checklist.getMessages():
                ET.SubElement(child, 'msg').text = str(msg)
    
    manager.indent(root)
    tree = ET.ElementTree(root)
    with open('data/checklists.xml', 'wb') as file:
        tree.write(file)
        file.close()
        
'''
Busca y actualiza el estado del objeto Checklist indicado en el XML
@param _id: identificador de la Checklist que se va a actualizar
@param state: nuevo estado de la checklist a actualizar
'''     
def updateState(_id, state):
    tree = ET.parse('data/checklists.xml')
    root = tree.getroot()
    for child in root.findall('checklist'):
        if child.find('id').text == str(_id):
            child.find('state').text = state
           
    manager.indent(root)
    tree = ET.ElementTree(root)
    with open('data/checklists.xml', 'wb') as file:
        tree.write(file)
        file.close()
   
'''
Elimina el objeto Checklist indicado del correspondiente XML
@param _id: identificador de la Checklist que se va a borrar
'''       
def delete(_id):
    tree = ET.parse('data/checklists.xml')
    root = tree.getroot()
    for child in root.findall('checklist'):
        if child.find('id').text == str(_id):
            root.remove(child)
    
    with open('data/checklists.xml', 'wb') as file:
        tree.write(file)
        file.close()