import xml.etree.ElementTree as ET
import os.path
from model.task import Task
import manager

'''
Inserta una tarea en el XML correspondiente como un nuevo elemento
@param task: objeto Task que se va a insertar
'''
def insert(task):
    if not os.path.isfile('data/tasks.xml'):
        root = ET.Element('tasks')
        
    else:
        tree = ET.parse('data/tasks.xml')
        root = tree.getroot()
        
    t = ET.SubElement(root, 'task')
    ET.SubElement(t, 'name').text = task.getName()
    ET.SubElement(t, 'idList').text = str(task.getIdList())
    if task.getOwner() != None:
        ET.SubElement(t, 'owner').text = str(task.getOwner())
    else:
        ET.SubElement(t, 'owner')
    ET.SubElement(t, 'checked').text = str(task.isChecked())
    
    manager.indent(root)
    tree = ET.ElementTree(root)
    with open('data/tasks.xml', 'wb') as file:
        tree.write(file)
        file.close()
 
'''
Busca y retorna todos los objetos Task almacenados en el XML
@return tasks: array de objetos Task con todas las tareas guardadas
'''
def findAll():
    tasks = {}
    if os.path.isfile('data/tasks.xml'):
        with open('data/tasks.xml', 'r') as file:
            tree = ET.parse(file)
            root = tree.getroot()
            for child in root.findall('task'):
                name = child.find('name').text
                idList = child.find('idList').text
                owner = child.find('owner')
                if owner != None:
                    idOwner = owner.text
                else:
                    idOwner = None
                checked = child.find('checked')
                task = Task(name, int(idList))
                task.setOwner(idOwner)
                task.setChecked(checked)
                tasks[(name, str(idList))] = task
               
            file.close()
        
    return tasks
    
'''
Busca y retorna el objeto Task con el nombre y id de checklist indicados de entre los almacenados en el XML
@param name: nombre de la tarea buscada
@param idList: identificador de la checklist a la que pertenece la tarea
@return task: objeto Task con la tarea coincidente
'''
def findByNameAndIdList(name, idList):
    task = None
    if os.path.isfile('data/tasks.xml'):
        with open('data/tasks.xml', 'r') as file:
            tree = ET.parse(file)
            root = tree.getroot()
            for child in root.findall('task'):
                if child.find('name').text == str(name) and child.find('idList').text == str(idList):
                    name = child.find('name').text
                    idList = child.find('idList').text
                    owner = child.find('owner')
                    if owner != None:
                        idOwner = owner.text
                    else:
                        idOwner = None
                    checked = child.find('checked').text
                    task = Task(name, int(idList))
                    task.setOwner(idOwner)
                    task.setChecked(checked)
                    break
               
            file.close()
            
    return task
    
'''
Busca y retorna los objetos Task pertenecientes a la checklist indicada del XML
@param idList: identificador de la checklist de la cual se buscan sus tareas
@return tasks: array de objetos Task con las tareas coincidentes
'''
def findAllByList(idList):
    tasks = []
    if os.path.isfile('data/tasks.xml'):
        with open('data/tasks.xml', 'r') as file:
            tree = ET.parse(file)
            root = tree.getroot()
            for child in root.findall('task'):
                if child.find('idList').text == str(idList):
                    name = child.find('name').text
                    idList = child.find('idList').text
                    owner = child.find('owner')
                    if owner != None:
                        idOwner = owner.text
                    else:
                        idOwner = None
                    checked = child.find('checked').text
                    task = Task(name, int(idList))
                    task.setOwner(idOwner)
                    task.setChecked(checked)
                    tasks.append(task)
                       
            file.close()
            
    return tasks
    
'''
Actualiza el objeto Task indicado del XML
@param task: objeto Task que se quiere actualizar
@param newName: nuevo nombre con el que se quiere almacenar la tarea, si es None no se modifica
'''    
def updateTask(task, newName):
    tree = ET.parse('data/tasks.xml')
    root = tree.getroot()
    for child in root.findall('task'):
        if child.find('name').text == task.getName() and child.find('idList').text == str(task.getIdList()):
            if newName != None:
                child.find('name').text = newName
                child.find('checked').text = 'False'
            if task.getOwner() != None:
                child.find('owner').text = str(task.getOwner())
            else:
                child.remove(child.find('owner'))
                ET.SubElement(child, 'owner')
    
    with open('data/tasks.xml', 'wb') as file:
        tree.write(file)
        file.close()

'''
Actualiza el estado del objeto Task indicado dentro del XML
@param task: objeto Task que se quiere actualizar
'''    
def updateTaskState(task):
    tree = ET.parse('data/tasks.xml')
    root = tree.getroot()
    for child in root.findall('task'):
        if child.find('name').text == task.getName() and child.find('idList').text == str(task.getIdList()):
            child.find('checked').text = str(task.isChecked())
    
    with open('data/tasks.xml', 'wb') as file:
        tree.write(file)
        file.close()

'''
Elimina el objeto Task con el nombre e identificador de checklist indicados del XML
@param name: nombre de la tarea a eliminar
@param idList: identificador de la Checklist a la que pertenece la tarea que se va a borrar
'''  
def delete(name, idList):
    tree = ET.parse('data/tasks.xml')
    root = tree.getroot()
    for child in root.findall('task'):
         if child.find('name').text == name and child.find('idList').text == str(idList):
            root.remove(child)
    
    with open('data/tasks.xml', 'wb') as file:
        tree.write(file)
        file.close()